function figh = eval_pi(cfg)
%EVAL_PI Evaluates PI estimation experiments, starting from results data file
%   FIGH = EVAL_PI(CFG)
% Inputs:
%   CFG     - experiment configuration, with fields:
%       datafile    - name of datafile to which results should be saved
%       styles      - linestyles for plotting the results of each experiment, cell array
% Outputs:
%   FIGH    - handle to figure created
%
% Loads "results" structure array from data file. Identifies automatically the algorithms and 
% experiment configurations, and plots for every algorithm and every value of k:
%   - the error in estimating PI
%   - the execution time with 95% confidence bounds as a shaded region
%
% See also run_pi, estimatepi

% Copyright:    2010 Delft Center for Systems & Control (TUDelft), and Lucian Busoniu
% Author:       Lucian Busoniu
% Version:      1.0
% 
% Please do not redistribute this code without permission from the authors. If you wish to
% redistribute, contact one of: Robert Babuska, Lucian Busoniu, Ivo Grondman, Gabriel Lopes


% provide for argument-less execution
if nargin < 1, cfg = struct; end;

% default config parameters
CFG.datafile = 'data/pidata';
CFG.styles = {'-k', '--r', ':g', '-.b'};

cfg = parsestruct(cfg, CFG);

% load the data
load(cfg.datafile, 'results');

% infer other experiment parameters
nexp = length(results);                 % number of experiments
nruns = size(results(1).exectime, 1);   % number of runs for each experiment
K = size(results(1).exectime, 2);       % maximum number of terms in the series

% iterate over each algorithm
figh = figure; set(figh, 'Position', [0, 0, 450, 700]); movegui(figh, 'center');
% set of handles for plot elements we want to mention in the legend
handles1 = nan(nexp, 1);        % first subplot: values
handles2 = nan(nexp+1, 1);      % second subplot: mean values + one item for confidence region
for i = 1:nexp,
    % in a first subplot, record the errors in estimating PI
    subplot(211);
    handles1(i) = semilogy(1:K, abs(results(i).piest - pi), cfg.styles{i}); hold on;
    % in a second subplot, show the mean+confidence level of the execution time, in milliseconds
    subplot(212);
    handles2(i) = semilogy(1:K, 1000*mean(results(i).exectime), cfg.styles{i}); hold on;
    % compute confidence bound using ttest
    [h, p, conf] = ttest(results(i).exectime);
    % negative confidence bounds are meaningless (and cannot be plotted on a logarithmic scale)
    conf = max(conf, 1e-5);
    % plot the confidence region as a shaded region, using patch; remember handle
    % the handle is overwritten at every iteration, that's not a problem as they all look the same
    handles2(nexp+1) = patch([1:K fliplr(1:K)], 1000*[conf(1, :) fliplr(conf(2, :))], ...
        -.1 * ones(2*K, 1), [.7 .7 .7], 'EdgeColor' , 'none');
end;

% add labels, legend etc.
subplot(211);
xlabel('K'); ylabel('Estimation error');
lh1 = legend(handles1, results(:).algorithm); 
set(lh1, 'FontSize', 8, 'Location', 'SouthEast');
subplot(212);
xlabel('K'); ylabel('Execution time [ms]');
lh2 = legend(handles2, results(:).algorithm, '95% confidence regions');
set(lh2, 'FontSize', 8, 'Location', 'NorthEast');

% Note 'ramanujan' actually converges to best floating point value of PI after the second iteration,
% so its error NOT visible on the plot!
% Also, 'chudnovsky' already reaches the best estimate within floating point precision from the first
% iteration, so the error remains constant afterwards

