function run_pi(cfg)
%RUN Runs PI estimation experiments, saves results
%   RUN_PI(CFG)
% Inputs:
%   CFG     - experiment configuration, with fields:
%       algorithms  - which algorithms to run (cell array of algorithm names)
%       nruns       - how many runs of each experiment to perform (to get statistical confidence)
%       K           - up to how many terms to expand in the series while computing PI
%       datafile    - name of datafile to which results should be saved
%
% The function will perform, for each algorithm, nruns with the number of terms in the series going
% from 1 to K. A "results" structure array will be saved, one element for each algorithm. The fields
% of this results structure are:
%       algorithm   - algorithm used, as a string
%       piest       - estimates of pi for each value of k, 1 x K array
%       exectime    - execution times for each value of k and run, nruns x K array
% The results will be saved to the configured datafile.
%
% See also eval_pi, estimatepi

% Copyright:    2010 Delft Center for Systems & Control (TUDelft), and Lucian Busoniu
% Author:       Lucian Busoniu
% Version:      1.0
% 
% Please do not redistribute this code without permission from the authors. If you wish to
% redistribute, contact one of: Robert Babuska, Lucian Busoniu, Ivo Grondman, Gabriel Lopes


% provide for argument-less execution (everything at default values)
if nargin < 1, cfg = struct; end;

% default config parameters
CFG.algorithms = {'madhava', 'newton', 'ramanujan', 'chudnovsky'};
CFG.nruns = 20;
CFG.K = 10; 
CFG.datafile = 'data/pidata';

cfg = parsestruct(cfg, CFG);

nexp = length(cfg.algorithms);  % number of experiments
results = struct;
% iterate over each algorithm
for i = 1:nexp,
    % initialize results fields
    results(i).algorithm = cfg.algorithms{i};       % save algorithm name on the results
    results(i).piest = nan(1, cfg.K);               % estimated values, one per k
    results(i).exectime = nan(cfg.nruns, cfg.K);    % init execution times, nruns per k
    
    fprintf('Running algorithm [%s] %d times.\n', cfg.algorithms{i}, cfg.nruns);
    
    params = struct;
    % run the algorithm for every k starting from 1 (not 0, for convenience)
    for k = 1:cfg.K,
        params.K = k;
        % run nexp times to get statistical confidence in execution time
        for irun = 1:cfg.nruns,
            [piest info] = estimatepi(cfg.algorithms{i}, params);
            results(i).exectime(irun, k) = info.exectime;
        end;
        % the values themselves are deterministic, so they only need to be saved once
        results(i).piest(k) = piest;
    end;
end;

save(cfg.datafile, 'results');